//---------------------------------------------------------------------
// System configuration
//
#ifndef FALSE
  #define FALSE  0
  #define TRUE   (!FALSE)
#endif

#define RC5_ENABLED  TRUE       // TRUE: add code for RC5 receiver


//---------------------------------------------------------------------
// Timing Definitions:
//
#ifndef F_CPU
   // prevent compiler error by supplying a default 
   # warning "F_CPU not defined for <main.c>", set default to 8MHz 
   # define F_CPU 8000000UL
   // if changed: check every place where it is used
   // (possible range underflow/overflow in preprocessor!)
#endif

#define TICK_PERIOD 10000L       // int every 10000us -> 100Hz


//---------------------------------------------------------------------
// Inter Process Communication
//

// #define Communicate    GPIOR0   // this was at Attiny2313

extern volatile unsigned char Communicate;

// volatile unsigned char Communicate;


#define C_Received        0    // a new IR command was received
                               //       - issued by ISR(Timer0)
                               //       - cleared by s88_load

#define C_S88_Sent        1    // a message was delivered

#define C_Tick            2    // a Tickevent happened
                               //       - not used

#define C_LED             3    // LED should be on
                                // issued by main
                                // queried by s88_isr to restore LED
  
extern volatile signed char timerval;    // is incremented by ISR

extern volatile unsigned int current_message;


//========================================================================

// This union allows to access 16 bits as word or as two bytes.
// This approach is (probably) more efficient than shifting.
union data16
{
    uint16_t asWord;
    uint8_t  asByte[2];
};



// 4.b) Useful inline code
//========================================================================

static inline unsigned char semaphor_query(unsigned char flag) 
       __attribute__((always_inline));

unsigned char semaphor_query(unsigned char flag)
  {
    return (Communicate & (1<<flag));
  }
               
static inline void semaphor_set(unsigned char flag) 
       __attribute__((always_inline));

void semaphor_set(unsigned char flag)
  {
    cli();
    Communicate |= (1<<flag);
    sei();
  }
       
static inline unsigned char semaphor_get(unsigned char flag) 
       __attribute__((always_inline));

unsigned char semaphor_get(unsigned char flag)
  {
    unsigned char value;
    cli();
    value = Communicate & (1<<flag);
    Communicate &= (~(1<<flag));
    sei();
    return(value);
  }
        

//------------------------------------------------------------------------
// Delay-Macro (all values in us) -> this is busy waiting
//------------------------------------------------------------------------
//
// same macro as in util/delay.h, but use up to some ms.
// The maximal possible delay is 262.14 ms / F_CPU in MHz.
// This is 16ms for 16MHz; longest used delay: 1000us

#ifndef _UTIL_DELAY_H_
  #include <util/delay.h>
#endif

static inline void _mydelay_us(double __us) __attribute__((always_inline));
void
_mydelay_us(double __us)
{
    uint16_t __ticks;
    double __tmp = ((F_CPU) / 4e6) * __us;
    if (__tmp < 1.0)
        __ticks = 1;
    else if (__tmp > 65535)
        __ticks = 0;    /* i.e. 65536 */
    else
        __ticks = (uint16_t)__tmp;
    _delay_loop_2(__ticks);
}   

